@@ -4,12 +4,12 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Common::Conf::ReConstants;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 
 sub compactConf {
     my ( $self, $conf ) = @_;
 
-    return $conf if ( $conf->{'dontCompactConf'} );
+    return $conf unless ( $conf->{'compactConf'} );
 
     # Remove unused auth parameters
     my %keep;
@@ -50,10 +50,12 @@ sub compactConf {
 
     # Disabled for now:
 
-    # Remove unused issuerDB parameters
+    # Remove unused issuerDB parameters except options
     foreach my $k ( keys %$issuerParameters ) {
         unless ( $conf->{ $k . "Activation" } ) {
-            delete $conf->{$_} foreach ( @{ $issuerParameters->{$k} } );
+            foreach ( @{ $issuerParameters->{$k} } ) {
+                delete $conf->{$_} unless ( $_ =~ /^issuers/ );
+            }
         }
     }
 
