@@ -4,11 +4,9 @@ use strict;
 use Lemonldap::NG::Common::UserAgent;
 use JSON qw(from_json);
 
-our $VERSION = '2.0.10';
-
+our $VERSION = '2.0.12';
 our $_ua;
 
-our $time;
 
 sub ua {
     return $_ua if ($_ua);
@@ -19,25 +17,23 @@ sub checkMaintenanceMode {
     my ( $class, $req ) = @_;
     my $vhost = $class->resolveAlias($req);
     $class->tsv->{lastVhostUpdate} //= {};
-    unless (
+    $class->_loadVhostConfig( $req, $vhost )
+      unless (
         $class->tsv->{defaultCondition}->{$vhost}
         and (
             time() - $class->tsv->{lastVhostUpdate}->{$vhost} <
-            $class->tsv->{checkTime} )
-      )
-    {
-        $class->loadVhostConfig( $req, $vhost );
-    }
+            $class->checkTime )
+      );
+
     return $class->Lemonldap::NG::Handler::Main::checkMaintenanceMode($req);
 }
 
-sub loadVhostConfig {
+sub _loadVhostConfig {
     my ( $class, $req, $vhost ) = @_;
     my $json;
     if ( $class->tsv->{useSafeJail} ) {
         my $rUrl = $req->{env}->{RULES_URL}
-          || (
-            (
+          || ( (
                 $class->localConfig->{loopBackUrl}
                 || "http://127.0.0.1:" . $req->{env}->{SERVER_PORT}
             )
@@ -65,9 +61,22 @@ q"I refuse to compile rules.json when useSafeJail isn't activated! Yes I know, I
     }
     $json->{rules} ||= { default => 1 };
     $json->{headers} //= { 'Auth-User' => '$uid' };
+
+    # Removed forbidden session attributes
+    foreach
+      my $v ( split /\s+/, $class->tsv->{hiddenAttributes} )
+    {
+        foreach ( keys %{ $json->{headers} } ) {
+            delete $json->{headers}->{$_}
+              if $json->{headers}->{$_} eq '$' . $v;
+        }
+    }
+
+    $class->logger->debug("DevOps handler called by $vhost");
     $class->locationRulesInit( undef, { $vhost => $json->{rules} } );
     $class->headersInit( undef,       { $vhost => $json->{headers} } );
     $class->tsv->{lastVhostUpdate}->{$vhost} = time;
+
     return;
 }
 
