@@ -1,6 +1,6 @@
 package Lemonldap::NG::Handler::Main::Reload;
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.12';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -57,9 +57,18 @@ sub checkConf {
             $class->logger->debug($Lemonldap::NG::Common::Conf::msg);
         }
     }
-    if ( $force or !$class->cfgNum or $class->cfgNum != $conf->{cfgNum} ) {
+    $Lemonldap::NG::Common::Conf::msg = '';
+
+    if (   $force
+        or !$class->cfgNum
+        or !$class->cfgDate
+        or $class->cfgNum != $conf->{cfgNum}
+        or $class->cfgDate != $conf->{cfgDate} )
+    {
         $class->logger->debug("Get configuration $conf->{cfgNum}");
-        unless ( $class->cfgNum( $conf->{cfgNum} ) ) {
+        unless ( $class->cfgNum( $conf->{cfgNum} )
+            && $class->cfgDate( $conf->{cfgDate} ) )
+        {
             $class->logger->error('No configuration available');
             return 0;
         }
@@ -78,7 +87,7 @@ sub checkConf {
             }
         }
     }
-    $class->tsv->{checkTime} = $conf->{checkTime} if ( $conf->{checkTime} );
+    $class->checkTime( $conf->{checkTime} ) if $conf->{checkTime};
     $class->lastCheck( time() );
     $class->logger->debug("$class: configuration is up to date");
     return 1;
@@ -164,7 +173,7 @@ sub configReload {
 
     foreach my $sub (
         qw( defaultValuesInit jailInit portalInit locationRulesInit
-        sessionStorageInit headersInit postUrlInit aliasInit )
+        sessionStorageInit headersInit postUrlInit aliasInit oauth2Init )
       )
     {
         $class->logger->debug("Process $$ calls $sub");
@@ -185,7 +194,8 @@ sub jailInit {
             multiValuesSeparator => $conf->{multiValuesSeparator},
         }
     );
-    $class->tsv->{jail}->build_jail( $class, $conf->{require} );
+    $class->tsv->{jail}
+      ->build_jail( $class, $conf->{require}, $conf->{requireDontDie} );
 }
 
 ## @imethod protected void defaultValuesInit(hashRef args)
@@ -200,7 +210,8 @@ sub defaultValuesInit {
         timeoutActivityInterval useRedirectOnError useRedirectOnForbidden
         useSafeJail             whatToTrace        handlerInternalCache
         handlerServiceTokenTTL  customToTrace      lwpOpts lwpSslOpts
-        authChoiceParam         authChoiceAuthBasic
+        authChoiceAuthBasic     authChoiceParam    hiddenAttributes
+        upgradeSession
         )
     );
 
@@ -214,7 +225,7 @@ sub defaultValuesInit {
         # Override with vhost options
         if ( $conf->{vhostOptions} ) {
             my $name = 'vhost' . ucfirst($opt);
-            foreach my $vhost ( keys %{ $conf->{vhostOptions} } ) {
+            foreach my $vhost ( sort keys %{ $conf->{vhostOptions} } ) {
                 $conf->{vhostOptions}->{$vhost} ||= {};
                 my $val = $conf->{vhostOptions}->{$vhost}->{$name};
 
@@ -228,13 +239,15 @@ sub defaultValuesInit {
         }
     }
     if ( $conf->{vhostOptions} ) {
-        foreach my $vhost ( keys %{ $conf->{vhostOptions} } ) {
+        foreach my $vhost ( sort keys %{ $conf->{vhostOptions} } ) {
             $class->tsv->{type}->{$vhost} =
               $conf->{vhostOptions}->{$vhost}->{vhostType};
             $class->tsv->{authnLevel}->{$vhost} =
               $conf->{vhostOptions}->{$vhost}->{vhostAuthnLevel};
             $class->tsv->{serviceTokenTTL}->{$vhost} =
               $conf->{vhostOptions}->{$vhost}->{vhostServiceTokenTTL};
+            $class->tsv->{accessToTrace}->{$vhost} =
+              $conf->{vhostOptions}->{$vhost}->{vhostAccessToTrace};
         }
     }
     return 1;
@@ -275,9 +288,23 @@ sub locationRulesInit {
     my ( $class, $conf, $orules ) = @_;
 
     $orules ||= $conf->{locationRules};
+    $class->tsv->{vhostReg} = [];
+    my @lastReg;
 
     foreach my $vhost ( keys %$orules ) {
         my $rules = $orules->{$vhost};
+        if ( $vhost =~ /[\%\*]/ ) {
+            my $expr = join '[^\.]*', map {
+                my $elt = $_;
+                join '.*', map { quotemeta $_ } split /\*/, $elt;
+            } split /\%/, $vhost;
+            if ($expr) {
+                push @{ $class->tsv->{vhostReg} }, [ qr/^$expr$/, $vhost ];
+            }
+            else {
+                push @lastReg, [ qr/.+/, $vhost ];
+            }
+        }
         $class->tsv->{locationCount}->{$vhost}         = 0;
         $class->tsv->{locationCondition}->{$vhost}     = [];
         $class->tsv->{locationProtection}->{$vhost}    = [];
@@ -321,12 +348,22 @@ sub locationRulesInit {
             $class->tsv->{defaultProtection}->{$vhost} = 0;
         }
     }
+    @{ $class->tsv->{vhostReg} } = sort {
+        my $av = $a->[1];
+        my $bv = $b->[1];
+        return 1  if $av =~ /^\*/ and $bv !~ /^\*/;
+        return -1 if $bv =~ /^\*/ and $av !~ /^\*/;
+        return 1  if $av =~ /^\%/ and $bv !~ /^\%/;
+        return -1 if $bv =~ /^\%/ and $av !~ /^\%/;
+        return length($bv) <=> length($av) || $av cmp $bv;
+    } @{ $class->tsv->{vhostReg} } if @{ $class->tsv->{vhostReg} };
+    push @{ $class->tsv->{vhostReg} }, @lastReg if @lastReg;
     return 1;
 }
 
 ## @imethod protected void sessionStorageInit(hashRef args)
 # Initialize the Apache::Session::* module choosed to share user's variables
-# and the Cache::Cache module choosed to cache sessions
+# and the Cache::Cache module chosen to cache sessions
 # @param $args reference to the configuration hash
 sub sessionStorageInit {
     my ( $class, $conf ) = @_;
@@ -363,13 +400,14 @@ sub sessionStorageInit {
         if ( $conf->{status} ) {
             my $params = "";
             if ( $class->tsv->{sessionCacheModule} ) {
-                $params = ' ' . join(
+                $params = $class->tsv->{sessionCacheModule} . ',{' . join(
                     ',',
-                    $class->tsv->{sessionCacheModule} . map {
-                        "$_ => "
-                          . $class->tsv->{sessionCacheOptions}->{$_}
-                    } keys %{ $class->tsv->{sessionCacheOptions} // {} }
-                );
+                    map {
+                        "$_ => '"
+                          . $class->tsv->{sessionCacheOptions}->{$_} . "'"
+                      }
+                      keys %{ $class->tsv->{sessionCacheOptions} // {} }
+                ) . '}';
             }
             $class->tsv->{statusPipe}->print("RELOADCACHE $params\n");
         }
@@ -527,9 +565,8 @@ sub conditionSub {
         eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
         return (
             sub {
-
                 my ($req) = @_;
-                $class->localUnlog;
+                $class->localUnlog( $req, @_ );
                 $req->{env}->{'psgi.r'}->add_output_filter(
                     sub {
                         my $r = $_[0]->r;
@@ -581,8 +618,28 @@ sub aliasInit {
 
 # TODO: support wildcards in aliases
 
+## @method arrayref oauth2Init
+# @param options vhostOptions configuration item
+# @return arrayref of vhost and aliases
+sub oauth2Init {
+    my ( $class, $conf ) = @_;
+
+    foreach my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} || {} } ) {
+        if (    $conf->{oidcRPMetaDataOptions}->{$rp}
+            and $conf->{oidcRPMetaDataOptions}->{$rp}
+            ->{oidcRPMetaDataOptionsClientID} )
+        {
+            $class->tsv->{oauth2Options}->{$rp}->{clientId} =
+              $conf->{oidcRPMetaDataOptions}->{$rp}
+              ->{oidcRPMetaDataOptionsClientID};
+        }
+    }
+    return 1;
+}
+
 sub substitute {
     my ( $class, $expr ) = @_;
+    $expr ||= '';
 
     # substitute special vars, just for retro-compatibility
     $expr =~ s/\$date\b/&date/sg;
@@ -595,6 +652,12 @@ sub substitute {
     $expr =~ s/\$env->\{/\$r->{env}->\{/g;
     $expr =~ s/\bskip\b/q\{999_SKIP\}/g;
 
+    # handle inGroup
+    $expr =~ s/\binGroup\(([^)]*)\)/listMatch(\$s->{'hGroups'},$1,1)/g;
+
+    # handle has2f
+    $expr =~ s/\bhas2f\(([^),]*)\)/has2f(\$s,$1)/g;
+
     return $expr;
 }
 
