@@ -9,6 +9,7 @@ use Lemonldap::NG::Manager::Build::CTrees;
 use Lemonldap::NG::Manager::Build::PortalConstants;
 use Lemonldap::NG::Manager::Conf::Zero;
 use Data::Dumper;
+use Regexp::Common 'URI';
 use Regexp::Assemble;
 use JSON;
 use Getopt::Std;
@@ -32,6 +33,7 @@ my $module          = __PACKAGE__;
 
 my @angularScopeVars;
 my @bool;
+my @arrayParam;
 my @cnodesKeys;
 my %cnodesRe;
 my @ignoreKeys;
@@ -282,8 +284,8 @@ $defaultAttr}
         oidcOPMetaDataJWKS oidcOPMetaDataOptions
         oidcOPMetaDataExportedVars oidcRPMetaDataOptions
         oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims
-        oidcRPMetaDataMacros casAppMetaDataExportedVars
-        casAppMetaDataOptions casAppMetaDataMacros
+        oidcRPMetaDataMacros oidcRPMetaDataScopeRules
+        casAppMetaDataExportedVars casAppMetaDataOptions casAppMetaDataMacros
         casSrvMetaDataExportedVars casSrvMetaDataOptions
         )
       )
@@ -295,6 +297,17 @@ $defaultAttr}
     my $confConstants =
       "our \$hashParameters = qr/^" . $ra->as_string . "\$/;\n";
     $ra = Regexp::Assemble->new;
+    foreach (@arrayParam) {
+        $ra->add($_);
+    }
+
+    # Not in Tree.pm
+    foreach (qw(mySessionAuthorizedRWKeys)) {
+        $ra->add($_);
+    }
+    $confConstants .=
+      "our \$arrayParameters = qr/^" . $ra->as_string . "\$/;\n";
+    $ra = Regexp::Assemble->new;
     foreach (@bool) {
         $ra->add($_);
     }
@@ -358,6 +371,7 @@ our %EXPORT_TAGS = (
           DEFAULTCONFBACKENDOPTIONS
           NO
           \$hashParameters
+          \$arrayParameters
           \@sessionTypes
           \$boolKeys
           )
@@ -453,6 +467,8 @@ sub buildPortalConstants() {
 
     printf STDERR $format, $self->portalConstantsFile;
     open( F, '>', $self->portalConstantsFile ) or die($!);
+    my $urire = $RE{URI}{HTTP}{ -scheme=>qr/https?/ }{-keep};
+    $urire =~ s/([\$\@])/\\$1/g;
     my $content = <<EOF;
 # This file is generated by $module. Don't modify it by hand
 package Lemonldap::NG::Portal::Main::Constants;
@@ -463,6 +479,7 @@ use Exporter 'import';
 our \$VERSION = '$Lemonldap::NG::Manager::Build::Attributes::VERSION';
 
 use constant HANDLER => 'Lemonldap::NG::Handler::PSGI::Main';
+use constant URIRE => qr{$urire};
 use constant {
 EOF
     for my $pe (
@@ -486,7 +503,7 @@ $portalConstsStr
 }
 
 # EXPORTER PARAMETERS
-our \@EXPORT_OK = ( 'portalConsts', 'HANDLER', $exports );
+our \@EXPORT_OK = ( 'portalConsts', 'HANDLER', 'URIRE', $exports );
 our %EXPORT_TAGS = ( 'all' => [ \@EXPORT_OK, 'import' ], );
 
 our \@EXPORT = qw(import PE_OK);
@@ -624,11 +641,11 @@ sub scanTree {
         else {
             # Get data type and build tree
             #
-            # Types : PerlModule bool boolOrExpr catAndAppList file hostname int
-            # keyTextContainer lmAttrOrMacro longtext openidServerList
-            # oidcAttributeContainer pcre rulesContainer samlAssertion
-            # samlAttributeContainer samlService select text trool url
-            # virtualHostContainer word password
+            # Types : PerlModule array bool boolOrExpr catAndAppList file
+            # hostname int keyTextContainer lmAttrOrMacro longtext
+            # openidServerList oidcAttributeContainer pcre rulesContainer
+            # samlAssertion samlAttributeContainer samlService select text
+            # trool url virtualHostContainer word password
 
             if ( $leaf =~ s/^\*// ) {
                 push @angularScopeVars, [ $leaf, "$path._nodes[$ord]" ];
@@ -681,9 +698,18 @@ sub scanTree {
                     }
                 }
             }
+
             if ($prefix) {
                 push @cnodesKeys, $leaf;
             }
+
+            # issue 2439
+            # FIXME: in future versions, oidcOPMetaDataJSON and samlIDPMetaDataXML shoud
+            # behave the same
+            if ( $leaf =~ /^oidcOPMetaData(?:JSON|JWKS)$/ ) {
+                push @simpleHashKeys, $leaf;
+            }
+
             if ( $attr->{type} =~ /^(?:catAndAppList|\w+Container)$/ ) {
                 $jleaf->{cnodes} = $prefix . $leaf;
                 unless ( $prefix or $leaf =~ $reIgnoreKeys ) {
@@ -700,6 +726,9 @@ sub scanTree {
                 if ( $attr->{type} eq 'bool' ) {
                     push @bool, $leaf;
                 }
+                if ( $attr->{type} eq 'array' ) {
+                    push @arrayParam, $leaf;
+                }
             }
             push @$json, $jleaf;
         }
@@ -977,7 +1006,7 @@ see L<Lemonldap::NG::Manager::Conf::Parser> before.
 =head1 DOWNLOAD
 
 Lemonldap::NG is available at
-L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
+L<https://lemonldap-ng.org/download>
 
 =head1 COPYRIGHT AND LICENSE
 
