@@ -8,21 +8,23 @@ package Lemonldap::NG::Portal::Auth::_WebForm;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_NOTOKEN
+  PE_FORMEMPTY
+  PE_FIRSTACCESS
   PE_CAPTCHAEMPTY
   PE_CAPTCHAERROR
-  PE_FIRSTACCESS
-  PE_FORMEMPTY
-  PE_NOTOKEN
-  PE_OK
-  PE_PASSWORDFORMEMPTY
   PE_TOKENEXPIRED
   PE_MALFORMEDUSER
+  PE_PASSWORDFORMEMPTY
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.10';
 
-extends 'Lemonldap::NG::Portal::Main::Auth',
-  'Lemonldap::NG::Portal::Lib::_tokenRule';
+extends qw(
+  Lemonldap::NG::Portal::Main::Auth
+  Lemonldap::NG::Portal::Lib::_tokenRule
+);
 
 has authnLevel => (
     is      => 'rw',
@@ -99,9 +101,11 @@ sub extractFormInfo {
     }
 
     # Security: check for captcha or token
-    if ( $self->captcha or $self->ottRule->( $req, {} ) ) {
+    if ( not $req->data->{'skipToken'}
+        and ( $self->captcha or $self->ottRule->( $req, {} ) ) )
+    {
         my $token;
-        unless ( $token = $req->param('token') ) {
+        unless ( $token = $req->param('token') or $self->captcha ) {
             $self->userLogger->error('Authentication tried without token');
             $self->ott->setToken($req);
             return PE_NOTOKEN;
@@ -136,7 +140,7 @@ sub extractFormInfo {
     # Other parameters
     $req->data->{timezone} = $req->param('timezone');
 
-    PE_OK;
+    return PE_OK;
 }
 
 # Set password in session data if wanted.
@@ -156,7 +160,7 @@ sub setAuthSessionInfo {
     # Store user timezone
     $req->{sessionInfo}->{'_timezone'} = $self->{'timezone'};
 
-    PE_OK;
+    return PE_OK;
 }
 
 # @return display type
