@@ -10,9 +10,10 @@ use Lemonldap::NG::Portal::Main::Constants ':all';
 use Encode;
 use Unicode::String qw(utf8);
 use Scalar::Util 'weaken';
+use IO::Socket::Timeout;
 use utf8;
 
-our $VERSION  = '2.0.6';
+our $VERSION  = '2.0.10';
 our $ppLoaded = 0;
 
 BEGIN {
@@ -47,26 +48,50 @@ sub new {
     }
     $self = Net::LDAP->new(
         \@servers,
-        onerror => undef,
+        onerror   => undef,
+        keepalive => 1,
         ( $conf->{ldapPort}    ? ( port    => $conf->{ldapPort} )    : () ),
         ( $conf->{ldapTimeout} ? ( timeout => $conf->{ldapTimeout} ) : () ),
         ( $conf->{ldapVersion} ? ( version => $conf->{ldapVersion} ) : () ),
         ( $conf->{ldapRaw}     ? ( raw     => $conf->{ldapRaw} )     : () ),
-        ( $conf->{caFile}      ? ( cafile  => $conf->{caFile} )      : () ),
-        ( $conf->{caPath}      ? ( capath  => $conf->{caPath} )      : () ),
+        ( $conf->{ldapCAFile}  ? ( cafile  => $conf->{ldapCAFile} )  : () ),
+        ( $conf->{ldapCAPath}  ? ( capath  => $conf->{ldapCAPath} )  : () ),
+        ( $conf->{ldapVerify}  ? ( verify  => $conf->{ldapVerify} )  : () ),
     );
     unless ($self) {
         $portal->logger->error($@);
         return 0;
     }
+    elsif ( $Net::LDAP::VERSION < '0.64' ) {
+
+        # CentOS7 has a bug in which IO::Socket::SSL will return a broken
+        # socket when certificate validation fails. Net::LDAP does not catch
+        # it, and the process ends up crashing.
+        # As a precaution, make sure the underlying socket is doing fine:
+        if (    $self->socket->isa('IO::Socket::SSL')
+            and $self->socket->errstr < 0 )
+        {
+            $portal->logger->error(
+                "SSL connection error: " . $self->socket->errstr );
+            return 0;
+        }
+    }
     bless $self, $class;
+
+    # Set socket timeouts
+    my $socket = $self->socket;
+    IO::Socket::Timeout->enable_timeouts_on($socket);
+    $socket->read_timeout( $conf->{ldapIOTimeout} );
+    $socket->write_timeout( $conf->{ldapIOTimeout} );
+
     if ($useTls) {
         my %h = split( /[&=]/, $tlsParam );
-        $h{cafile} = $conf->{caFile} if ( $conf->{caFile} );
-        $h{capath} = $conf->{caPath} if ( $conf->{caPath} );
+        $h{cafile} ||= $conf->{ldapCAFile} if ( $conf->{ldapCAFile} );
+        $h{capath} ||= $conf->{ldapCAPath} if ( $conf->{ldapCAPath} );
+        $h{verify} ||= $conf->{ldapVerify} if ( $conf->{ldapVerify} );
         my $mesg = $self->start_tls(%h);
         if ( $mesg->code ) {
-            $portal->logger->error('StartTLS failed');
+            $portal->logger->error( 'StartTLS failed: ' . $mesg->error );
             return 0;
         }
     }
@@ -180,7 +205,8 @@ sub userBind {
         # Return direct unless control resonse
         unless ( defined $resp ) {
             if ( $mesg->code == 49 ) {
-                $self->{portal}->userLogger->warn("Bad password");
+                $self->{portal}->userLogger->warn(
+                    "Bad password for $req->{user} (" . $req->address . ")" );
                 return PE_BADCREDENTIALS;
             }
             elsif ( $mesg->code == 0 ) {
@@ -215,6 +241,9 @@ sub userBind {
 
             # Get expiration warning and graces
             if ( $resp->grace_authentications_remaining ) {
+                $self->{portal}->logger->debug(
+                    "LDAP password policy - grace authentications remaining: "
+                      . $resp->grace_authentications_remaining );
                 $req->info(
                     $self->{portal}->loadTemplate(
                         $req,
@@ -227,14 +256,20 @@ sub userBind {
             }
 
             if ( $resp->time_before_expiration ) {
+                $self->{portal}->logger->debug(
+                    "LDAP password policy - time before expiration: "
+                      . $resp->time_before_expiration );
                 $req->info(
                     $self->{portal}->loadTemplate(
                         $req,
                         'simpleInfo',
                         params => {
-                            trspan => 'authRemaining,'
-                              . $self->convertSec(
-                                $resp->time_before_expiration
+                            trspan => 'pwdWillExpire,'
+                              . join(
+                                ',',
+                                $self->convertSec(
+                                    $resp->time_before_expiration
+                                )
                               )
                         }
                     )
@@ -253,24 +288,26 @@ sub userBind {
             $req->data->{ldapError} = $mesg->error;
         }
     }
-    $self->{portal}->userLogger->warn("Bad password for $req->{user}");
+    $self->{portal}->userLogger->warn(
+        "Bad password for $req->{user} (" . $req->address . ")" );
     return PE_BADCREDENTIALS;
 }
 
-## @method int userModifyPassword(string dn, string newpassword, string oldpassword, boolean ad)
+## @method int userModifyPassword(string dn, string newpassword, string oldpassword, boolean ad, boolean requireOldPassword)
 # Change user's password.
 # @param $dn DN
 # @param $newpassword New password
 # @param $oldpassword Current password
 # @param $ad Active Directory mode
+# @param $requireOldPassword Old password is needed to update
 # @return Lemonldap::NG::Portal constant
 sub userModifyPassword {
-    my ( $self, $dn, $newpassword, $oldpassword, $ad ) = @_;
-    my $ppolicyControl     = $self->{conf}->{ldapPpolicyControl};
-    my $setPassword        = $self->{conf}->{ldapSetPassword};
-    my $asUser             = $self->{conf}->{ldapChangePasswordAsUser};
-    my $requireOldPassword = $self->{conf}->{portalRequireOldPassword};
-    my $passwordAttribute  = "userPassword";
+    my ( $self, $dn, $newpassword, $oldpassword, $ad, $requireOldPassword ) =
+      @_;
+    my $ppolicyControl    = $self->{conf}->{ldapPpolicyControl};
+    my $setPassword       = $self->{conf}->{ldapSetPassword};
+    my $asUser            = $self->{conf}->{ldapChangePasswordAsUser};
+    my $passwordAttribute = "userPassword";
     my $err;
     my $mesg;
 
@@ -418,7 +455,7 @@ sub userModifyPassword {
             return PE_LDAPERROR;
         }
 
-        $self->{portal}->userLogger->notice("Password changed for $dn");
+        $self->{portal}->logger->notice("Password changed for $dn");
 
         # Rebind as manager for next LDAP operations if we were bound as user
         $self->bind() if $asUser;
@@ -551,8 +588,7 @@ sub userModifyPassword {
         return PE_WRONGMANAGERACCOUNT
           if ( $mesg->code == 50 || $mesg->code == 8 );
         if ( $mesg->code == 0 ) {
-            $self->{portal}
-              ->userLogger->notice("Password changed $self->{portal}->{user}");
+            $self->{portal}->logger->notice("Password changed for $dn");
 
            # Rebind as manager for next LDAP operations if we were bound as user
             $self->bind() if $asUser;
@@ -669,6 +705,9 @@ sub searchGroups {
 
                 # Launch group search
                 if ($group_value) {
+                    if ( $self->{conf}->{ldapGroupDecodeSearchedValue} ) {
+                        utf8::decode($group_value);
+                    }
 
                     if ( $dupcheck->{$group_value} ) {
                         $self->{portal}->logger->debug(
