@@ -5,12 +5,10 @@
 # Slave common functions
 package Lemonldap::NG::Portal::Lib::Slave;
 
-use Exporter;
-use base qw(Exporter);
 use strict;
+use Mouse;
 
-our @EXPORT  = qw(checkIP checkHeader);
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.10';
 
 # RUNNING METHODS
 
@@ -38,7 +36,8 @@ sub checkHeader {
     my $slave_header = 'HTTP_' . uc( $self->{conf}->{slaveHeaderName} );
     $slave_header =~ s/\-/_/g;
     my $headerContent = $req->env->{$slave_header};
-    $self->logger->debug("Required Slave header => $self->{conf}->{slaveHeaderName}");
+    $self->logger->debug(
+        "Required Slave header => $self->{conf}->{slaveHeaderName}");
     $self->logger->debug("Received Slave header content => $headerContent");
     return 1
       if (  $headerContent
