@@ -22,7 +22,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.12';
 
 # PROPERTIES
 
@@ -212,6 +212,8 @@ sub storeRequest {
     my $info = {};
     $info->{content} = $req->content;
     foreach ( keys %{ $req->env } ) {
+        next if $_ eq "psgi.errors";
+        next if $_ eq "psgi.input";
         $info->{$_} = $req->env->{$_} unless ( ref $req->env->{$_} );
     }
     return $self->_ott->createToken($info);
@@ -394,7 +396,7 @@ L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 =head1 DOWNLOAD
 
 Lemonldap::NG is available at
-L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
+L<https://lemonldap-ng.org/download>
 
 =head1 COPYRIGHT AND LICENSE
 
