@@ -5,8 +5,9 @@ package Lemonldap::NG::Portal::Main::Menu;
 use strict;
 use Mouse;
 use Clone 'clone';
+use Lemonldap::NG::Portal::Main::Constants 'URIRE';
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.12';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -30,7 +31,8 @@ has menuModules => (
     }
 );
 
-has specific => ( is => 'rw', default => sub { {} } );
+has specific      => ( is => 'rw', default => sub { {} } );
+has sfManagerRule => ( is => 'rw', default => sub { 1 } );
 
 has imgPath => (
     is      => 'rw',
@@ -44,7 +46,13 @@ has imgPath => (
 # INITIALIZATION
 
 sub init {
-    1;
+    my ($self) = @_;
+
+    $self->sfManagerRule(
+        $self->p->buildRule( $self->conf->{sfManagerRule}, 'sfManagerRule' ) );
+    $self->sfManagerRule(1) unless $self->sfManagerRule;
+
+    return 1;
 }
 
 # RUNNING METHODS
@@ -109,29 +117,42 @@ sub params {
     $res{AUTH_ERROR_TYPE} =
       $req->error_type( $res{AUTH_ERROR} = $req->menuError );
 
-    # Display menu 2fRegisters link only if at least a 2F device is registered
+# Display menu 2fRegisters link only if at least a 2F device is registered and rule
     $res{sfaManager} =
-      $self->p->_sfEngine->display2fRegisters( $req, $req->userData );
+         $self->p->_sfEngine->display2fRegisters( $req, $req->userData )
+      && $self->sfManagerRule->( $req, $req->userData );
     $self->logger->debug("Display 2fRegisters link") if $res{sfaManager};
 
-    # Display ContextSwitching link only if allowed
-    my $cswPlugin = $self->p->loadedModules->{
-        'Lemonldap::NG::Portal::Plugins::ContextSwitching'};
-    $res{contextSwitching} =
-        $cswPlugin
-      ? $cswPlugin->displaySwitchContext( $req, $req->userData )
-      : '';
-    $self->logger->debug("Display SwitchContext link -> $res{contextSwitching}")
-      if $res{contextSwitching};
+    # Display refresh my rights unless disabled
+    $res{RefreshMyRights} = $self->conf->{portalDisplayRefreshMyRights};
 
-    # Display DecryptValue link if allowed
-    my $dvPlugin =
-      $self->p->loadedModules->{'Lemonldap::NG::Portal::Plugins::DecryptValue'};
-    $res{decryptValue} =
-        $dvPlugin
-      ? $dvPlugin->displayLink( $req, $req->userData )
-      : '';
-    $self->logger->debug("Display DecryptValue link") if $res{decryptValue};
+    # Display menu links only if required
+    foreach (qw(ContextSwitching DecryptValue Notifications)) {
+        my $plugin =
+          $self->p->loadedModules->{"Lemonldap::NG::Portal::Plugins::$_"};
+        $res{$_} =
+            $plugin
+          ? $plugin->displayLink( $req, $req->userData )
+          : '';
+        my $msg = "Display $_ link";
+        $msg .= " -> $res{ContextSwitching}"
+          if ( $_ eq 'ContextSwitching' && $res{$_} );
+        $self->logger->debug($msg) if $res{$_};
+        undef $plugin;
+    }
+
+    # Decide whether to display the dropdown or regular text
+    $res{DropdownMenu} = 0;
+
+    foreach (
+        qw(RefreshMyRights sfaManager Notifications DecryptValue ContextSwitching)
+      )
+    {
+        if ( $res{$_} ) {
+            $res{DropdownMenu} = 1;
+            last;
+        }
+    }
 
     return %res;
 }
@@ -277,6 +298,7 @@ sub _buildApplicationHash {
     my $appuri  = $apphash->{options}->{uri}  || "";
     my $appdesc = $apphash->{options}->{description};
     my $applogo = $apphash->{options}->{logo};
+    my $apptip = $apphash->{options}->{tooltip} || $appname;
 
     # Detect sub applications
     my $subapphash;
@@ -310,6 +332,7 @@ sub _buildApplicationHash {
         appdesc     => $appdesc,
         applogo     => $applogo,
         appid       => $appid,
+        apptip      => $apptip,
     };
     $applicationHash->{applications} = $applications if $applications;
     return $applicationHash;
@@ -371,9 +394,8 @@ sub _filterHash {
             # Check rights
             my $appdisplay = $apphash->{$key}->{options}->{display}
               || "auto";
-            my ( $vhost, $appuri ) =
-              $apphash->{$key}->{options}->{uri} =~ m#^https?://([^/]*)(.*)#;
-            $vhost =~ s/:\d+$//;
+            $apphash->{$key}->{options}->{uri} =~ URIRE;
+            my ( $vhost, $appuri ) = ( $3, $5 );
             $vhost = $self->p->HANDLER->resolveAlias($vhost);
             $appuri ||= '/';
 
