@@ -9,12 +9,13 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.0.13';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
 use URI::Escape;
+use URI;
 use JSON;
 use Lemonldap::NG::Common::Util qw(getPSessionID);
 
@@ -575,14 +576,16 @@ sub updateSession {
     if ($id) {
 
         # Update sessionInfo data
-        ## sessionInfo updated if $id defined : quite strange !!
+        ## sessionInfo updated if $id defined : quite strange!!
         ## See https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues/430
         $self->logger->debug("Update session $id");
         foreach ( keys %$infos ) {
             $self->logger->debug("Update sessionInfo $_");
             $self->_dump( $infos->{$_} );
-            $req->{sessionInfo}->{$_} = $self->HANDLER->data->{$_} =
-              $infos->{$_};
+            $req->{sessionInfo}->{$_} = $infos->{$_};
+            if ( $self->HANDLER->data->{_session_id} && $id eq $self->HANDLER->data->{_session_id} ) {
+                $self->HANDLER->data->{$_} = $infos->{$_};
+            }
         }
 
         # Update session in global storage with _updateTime
@@ -874,33 +877,34 @@ sub sendHtml {
       'Pragma'        => 'no-cache',                               # HTTP 1.0
       'Expires'       => '0';                                      # Proxies
 
-    if ( $self->conf->{corsEnabled} ) {
-        my @cors = split /;/, $self->cors;
-        push @{ $res->[1] }, @cors;
-        $self->logger->debug('Apply following CORS policy :');
-        $self->logger->debug(" $_") for @cors;
-    }
+    $self->setCorsHeaderFromConfig($res);
 
     # Set authorized URL for POST
     my $csp = $self->csp . "form-action " . $self->conf->{cspFormAction};
     if ( my $url = $req->urldc ) {
         $self->logger->debug("Required urldc : $url");
-        $url =~ s#(https?://[^/]+).*#$1#;
+        $url =~ URIRE;
+        $url = $2 . '://' . $3 . ( $4 ? ":$4" : '' );
         $self->logger->debug("Set CSP form-action with urldc : $url");
         $csp .= " $url";
     }
     my $url = $args{params}->{URL};
     if ( defined $url ) {
         $self->logger->debug("Required Params URL : $url");
-        if ( $url =~ s#(https?://[^/]+).*#$1# ) {
+        if ( $url =~ URIRE ) {
+            $url = $2 . '://' . $3 . ( $4 ? ":$4" : '' );
             $self->logger->debug("Set CSP form-action with Params URL : $url");
             $csp .= " $url";
         }
     }
-    if ( defined $req->{cspFormAction} ) {
-        $self->logger->debug(
-            "Set CSP form-action with request URL: " . $req->{cspFormAction} );
-        $csp .= " " . $req->{cspFormAction};
+    if ( defined $req->data->{cspFormAction}
+        and ref( $req->data->{cspFormAction} ) eq "HASH" )
+    {
+        my $request_csp_form_action =
+          join( " ", keys %{ $req->data->{cspFormAction} } );
+        $self->logger->debug( "Set CSP form-action with request URL: "
+              . $request_csp_form_action );
+        $csp .= " " . $request_csp_form_action;
     }
 
     # Set SAML Discovery Protocol in form-action
@@ -930,11 +934,18 @@ sub sendHtml {
     }
 
     # Check if frames need to be embedded
+    # FIXME: we should use $req->data->{cspChildSrc} anywhere an iframe is
+    # created in the code, and remove this
     my @url;
     if ( $req->info ) {
         @url = map { s#https?://([^/]+).*#$1#; $_ }
           ( $req->info =~ /<iframe.*?src="(.*?)"/sg );
     }
+
+    # Update child-src header from request data
+    if ( ref( $req->data->{cspChildSrc} ) eq "HASH" ) {
+        push @url, keys %{ $req->data->{cspChildSrc} };
+    }
     if (@url) {
         $csp .= join( ' ', 'child-src', @url, "'self'" ) . ';';
     }
@@ -1037,9 +1048,7 @@ sub tplParams {
 }
 
 sub registerLogin {
-
-    # $user passed by BruteForceProtection plugin
-    my ( $self, $req, $uid ) = @_;
+    my ( $self, $req ) = @_;
     return
       unless ( $self->conf->{loginHistoryEnabled}
         and defined $req->authResult );
@@ -1069,13 +1078,12 @@ sub registerLogin {
                 }
             }
         }
-        $self->updatePersistentSession( $req, { 'loginHistory' => undef },
-            $uid );
+        $self->updatePersistentSession( $req, { 'loginHistory' => undef } );
         delete $req->sessionInfo->{loginHistory};
     }
 
     my $history = $req->sessionInfo->{_loginHistory} ||= {};
-    my $type    = ( $req->authResult > 0 ? 'failed' : 'success' ) . 'Login';
+    my $type = ( $req->authResult > 0 ? 'failed' : 'success' ) . 'Login';
     $history->{$type} ||= [];
     $self->logger->debug("Current login saved into $type");
 
@@ -1095,7 +1103,7 @@ sub registerLogin {
       if ( scalar @{ $history->{$type} } > $self->conf->{ $type . "Number" } );
 
     # Save into persistent session
-    $self->updatePersistentSession( $req, { _loginHistory => $history }, $uid );
+    $self->updatePersistentSession( $req, { _loginHistory => $history, } );
 
     PE_OK;
 }
@@ -1118,13 +1126,11 @@ sub _sumUpSession {
 sub corsPreflight {
     my ( $self, $req ) = @_;
     my @headers;
-    if ( $self->conf->{corsEnabled} ) {
-        my @cors = split /;/, $self->cors;
-        push @headers, @cors;
-        $self->logger->debug('Apply following CORS policy :');
-        $self->logger->debug(" $_") for @cors;
-    }
-    return [ 204, \@headers, [] ];
+    my $res = [ 204, \@headers, [] ];
+
+    $self->setCorsHeaderFromConfig($res);
+
+    return $res;
 }
 
 sub sendJSONresponse {
@@ -1153,11 +1159,8 @@ sub sendJSONresponse {
           "Access-Control-Allow-Credentials" => "true";
 
     }
-    elsif ( $self->conf->{corsEnabled} ) {
-        my @cors = split /;/, $self->cors;
-        push @{ $res->[1] }, @cors;
-        $self->logger->debug('Apply following CORS policy :');
-        $self->logger->debug(" $_") for @cors;
+    else {
+        $self->setCorsHeaderFromConfig($res);
     }
     return $res;
 }
@@ -1165,13 +1168,21 @@ sub sendJSONresponse {
 sub sendRawHtml {
     my ($self) = $_[0];
     my $res = Lemonldap::NG::Common::PSGI::sendRawHtml(@_);
+
+    $self->setCorsHeaderFromConfig($res);
+
+    return $res;
+}
+
+sub setCorsHeaderFromConfig {
+    my ( $self, $response ) = @_;
+
     if ( $self->conf->{corsEnabled} ) {
         my @cors = split /;/, $self->cors;
-        push @{ $res->[1] }, @cors;
+        push @{ $response->[1] }, @cors;
         $self->logger->debug('Apply following CORS policy :');
         $self->logger->debug(" $_") for @cors;
     }
-    return $res;
 }
 
 # Temlate loader
@@ -1198,4 +1209,16 @@ sub loadTemplate {
     return $tpl->output;
 }
 
+# This method extracts the scheme://host:port part of a URL for use in
+# Content-Security-Polity header
+sub cspGetHost {
+    my ( $self, $url ) = @_;
+    my $uri = $url // "";
+    unless ( $uri->isa("URI") ) {
+        $uri = URI->new($uri);
+    }
+    return (
+        $uri->scheme . "://" . ( $uri->_port ? $uri->host_port : $uri->host ) );
+}
+
 1;
