@@ -2,9 +2,12 @@ package Lemonldap::NG::Portal::Plugins::ForceAuthn;
 
 use strict;
 use Mouse;
-use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_MUSTAUTHN);
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_MUSTAUTHN
+);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.10';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
@@ -21,12 +24,12 @@ sub run {
     if (    $req->env->{HTTP_HOST}
         and $self->conf->{portal} =~ /\Q$req->{env}->{HTTP_HOST}/ )
     {
-        my $delta = time() - $req->{sessionInfo}->{_utime};
+        my $delta = time - $req->{sessionInfo}->{_utime};
         $self->logger->debug( "Delta with last Authn -> " . $delta );
 
-        $delta <= $self->conf->{portalForceAuthnInterval}
-          ? return PE_OK
-          : return PE_MUSTAUTHN;
+        return $delta <= $self->conf->{portalForceAuthnInterval}
+          ? PE_OK
+          : PE_MUSTAUTHN;
     }
 }
 
