@@ -13,15 +13,20 @@ package Lemonldap::NG::Portal::Plugins::SOAPServer;
 
 use strict;
 use Mouse;
-use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_FORMEMPTY);
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_FORMEMPTY
+  URIRE
+);
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.12';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Common::Conf::AccessLib';
+extends qw(
+  Lemonldap::NG::Portal::Main::Plugin
+  Lemonldap::NG::Common::Conf::AccessLib
+);
 
 has server => ( is => 'rw' );
-
 has configStorage => (
     is      => 'ro',
     lazy    => 1,
@@ -29,7 +34,6 @@ has configStorage => (
         $_[0]->{p}->HANDLER->localConfig->{configStorage};
     }
 );
-
 has exportedAttr => (
     is      => 'rw',
     lazy    => 1,
@@ -48,9 +52,13 @@ has exportedAttr => (
             if ( my $exportedAttr = $conf->{exportedAttr} ) {
                 $exportedAttr =~ s/^\s*\+\s+//;
                 @attributes = ( @attributes, split( /\s+/, $exportedAttr ) );
+
+                # Convert @attributes into hash to remove duplicates
+                my %attributes = map( { $_ => 1 } @attributes );
+                return '[' . join( ',', keys %attributes ) . ']';
             }
 
-            # convert @attributes into hash to remove duplicates
+            # Convert @attributes into hash to remove duplicates
             my %attributes = map( { $_ => 1 } @attributes );
             %attributes =
               ( %attributes, %{ $conf->{exportedVars} }, %{ $conf->{macros} },
@@ -112,29 +120,33 @@ sub init {
         $self->addUnauthRoute(
             sessions => { '*' => 'unauthSessions' },
             ['POST']
-        );
-        $self->addUnauthRoute(
+          )
+
+          ->addUnauthRoute(
             adminSessions => 'unauthAdminSessions',
             ['POST']
-        );
-        $self->addAuthRoute(
+          )
+
+          ->addAuthRoute(
             sessions => { '*' => 'badSoapRequest' },
             ['POST']
-        );
-        $self->addAuthRoute(
+          )
+
+          ->addAuthRoute(
             adminSessions => { '*' => 'badSoapRequest' },
             ['POST']
-        );
+          );
     }
     if ( $self->conf->{soapConfigServer} ) {
-        $self->addUnauthRoute( config => { '*' => 'config' }, ['POST'] );
-        $self->addAuthRoute( config => { '*' => 'badSoapRequest' }, ['POST'] );
+        $self->addUnauthRoute( config => { '*' => 'config' }, ['POST'] )
+          ->addAuthRoute( config => { '*' => 'badSoapRequest' }, ['POST'] );
     }
     if ( $self->conf->{wsdlServer} ) {
-        $self->addUnauthRoute( 'portal.wsdl' => 'getWsdl', ['GET'] );
-        $self->addAuthRoute( 'portal.wsdl' => 'getWsdl', ['GET'] );
+        $self->addUnauthRoute( 'portal.wsdl' => 'getWsdl', ['GET'] )
+          ->addAuthRoute( 'portal.wsdl' => 'getWsdl', ['GET'] );
     }
-    1;
+
+    return 1;
 }
 
 # SOAP DISPATCHERS
@@ -447,7 +459,7 @@ sub isAuthorizedURI {
     my ( $self, $req, $id, $url ) = @_;
     die 'id is required'  unless ($id);
     die 'uri is required' unless ($url);
-    die 'Bad uri'         unless ( $url =~ m#^https?://([^/]+)(/.*)?$# );
+    die 'Bad uri'         unless ( $url =~ URIRE );
     my ( $host, $uri ) = ( $1, $2 );
 
     # Get user session.
