@@ -4,7 +4,7 @@ package Lemonldap::NG::Portal::UserDB::AD;
 use strict;
 use Mouse;
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.11';
 
 extends 'Lemonldap::NG::Portal::UserDB::LDAP';
 
@@ -22,4 +22,14 @@ has filter => (
     }
 );
 
+has findUserFilter => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => sub {
+        $_[0]->conf->{AuthLDAPFilter}
+          || $_[0]->conf->{LDAPFilter}
+          || '(&(sAMAccountName=$user)(objectClass=person))';
+    }
+);
+
 1;
